import { throwsError, shuffle, preloadFiles } from '../src/utils';

function compare(arr1: string[], arr2: string[]): boolean {
  for (let i = 0; i < arr1.length; i++) {
    if (arr1[i] !== arr2[i]) {
      return false;
    }
  }

  return true;
}

function unique(arr: string[]): boolean {
  return arr.length === new Set(arr.map(item => item)).size;
}

describe('utils', () => {
  describe('throwsError', () => {
    it('should throw an error using the message passed by parameter', () => {
      const error = 'sorry, something failed';
      expect(() => throwsError(error)).toThrowError(`\`ts-audio\`: ${error}`);
    });
  });

  describe('shuffle', () => {
    it('should shuffle the order of the elements in a given array', () => {
      const files = [
        './file2.mp3',
        './file1.mp3',
        './file3.mp3',
        './file100.mp3',
        './file50.mp3',
      ];

      const shuffled = shuffle(files);

      expect(shuffled.length).toBe(files.length);
      expect(compare(shuffled, files)).toBeFalsy();
      expect(unique(shuffled)).toBeTruthy();
    });
  });

  describe('preload files', () => {
    const files = [
      './file2.mp3',
      './file1.mp3',
      './file3.mp3',
      './file4.mp3',
      './file5.mp3',
      './file6.mp3',
      './file7.mp3',
    ];

    it('should request up to 3 files concurrently', done => {
      let counter = 0;

      const api = () => {
        counter++;
        return Promise.resolve({} as Response);
      };

      preloadFiles(files, 3, api, () => {
        expect(counter).toBe(7);
        done();
      });
    });

    it('should request next file from the queue in case a request has failed', done => {
      let counter = 0;

      const api = (file: string) => {
        counter++;

        if (['./file3.mp3', './file7.mp3'].includes(file)) {
          return Promise.reject({} as Response);
        }

        return Promise.resolve({} as Response);
      };

      preloadFiles(files, 3, api, () => {
        expect(counter).toBe(7);
        done();
      });
    });
  });
});
